@@ -88,7 +88,7 @@ class OrderedModelQuerySet(models.QuerySet):
             update_kwargs.update(extra_kwargs)
         return self.update(**update_kwargs)
 
-    def bulk_create(self, objs, batch_size=None):
+    def bulk_create(self, objs, *args, **kwargs):
         order_field_name = self._get_order_field_name()
         order_with_respect_to = self.model.order_with_respect_to
         objs = list(objs)
@@ -109,7 +109,7 @@ class OrderedModelQuerySet(models.QuerySet):
         else:
             for order, obj in enumerate(objs, self.get_next_order()):
                 setattr(obj, order_field_name, order)
-        return super().bulk_create(objs, batch_size=batch_size)
+        return super().bulk_create(objs, *args, **kwargs)
 
     def _get_order_with_respect_to_filter_kwargs(self, ref):
         order_with_respect_to = self._get_order_with_respect_to()
@@ -154,11 +154,13 @@ class OrderedModelBase(models.Model):
 
     def _validate_ordering_reference(self, ref):
         if self.order_with_respect_to is not None:
-            self_kwargs = self._meta.default_manager._get_order_with_respect_to_filter_kwargs(
-                self
+            self_kwargs = (
+                self._meta.default_manager._get_order_with_respect_to_filter_kwargs(
+                    self
+                )
             )
-            ref_kwargs = ref._meta.default_manager._get_order_with_respect_to_filter_kwargs(
-                ref
+            ref_kwargs = (
+                ref._meta.default_manager._get_order_with_respect_to_filter_kwargs(ref)
             )
             if self_kwargs != ref_kwargs:
                 raise ValueError(
